--[[ This program is freeware; you may give it away and/or change it
under the GNU General Public License.  However, there is no
warrenty; not even an implied one.  When I say warrenty, you
say, "None!"  Also of note is that only the source code is
licensed under the GNU General Public License.  The art cannot
be edited, changed, or included with other programs.

The other software that this program uses is under a copyright
from the people whom wrote it, mostly being a variation on the
zlib license or BSD license.
For more information on the license, check the COPYING file
included with this program, or look on the GNU website,
www.gnu.org ]]

--[[
  What is contained in this file:
  * Bullet tables for both the hero and enemies.
  * Code for keeping track of bullets.
]]

lhc = require 'libs/hardoncollider'

Weapons = {}
local PI_DIVIDED_BY_180 = 0.017453292519943  -- This is just an approximation,
					     -- but it's close enough for me.
function Weapons.new()

  -- Put them all in here.
  local group = {}

  -- Bullets table.
  group.bullet = {}

-- Code taken from the game SHMUP.
--  "x" is the x-component, "y" is the y-component, "s" is for the speed,
--  "a" is for the angle from where it fires, and "w" is for its intended victim.
--  All parameters are madatory except for the speed and angle.
  function group.spawnBullet(x,y,s,a,w)
    -- Check the parameters to make sure they are the right types.
    assert(type(x) == "number", "Please pass a number for the first argument.")
    assert(type(y) == "number", "Please pass a number for the second argument.")
    assert(type(w) == "string", "Please pass the name of the intended victim as the last argument.")
    local count = #group.bullet+1
    group.bullet[count] = { x = x, y = y, speed = s or 100, angle = a or 90, dead = false, who = w, circle = lhc.addPoint(x,y) }
    assert(group.bullet[count].circle, "Sorry, shape wasn't created.")
    -- Add this to the "bullets" group for better collision detection.
    lhc.addToGroup("bullets", group.bullet[count].circle)
    -- Looks like the bullet has someone's name on it.
    if w == "hamster" then
      group.bullet[count].circle.who = "hamster"
    elseif w == "enemy" then
      group.bullet[count].circle.who = "enemy"
    end
  end

  -- Use this function to give the bullet table to another
  -- class.
  function group.getBulletTable()
    return group.bullet
  end

  -- Use this for the player's weapon settings.
  group.weapon = {}

  -- "pea-shooter" gun
  group.weapon[1] = { fireWeapon = function(hero_x, hero_y)
    -- These use half of the width and height.
    group.spawnBullet(hero_x-32, hero_y-35, 700, 270, "enemy")
  end ,
  inUse = true
  }

  -- Spreader gun
  group.weapon[2] = { fireWeapon = function(hero_x, hero_y)
   local speed = 700
   group.spawnBullet(hero_x, hero_y, speed, 285, "enemy")
   group.spawnBullet(hero_x, hero_y, speed, 275, "enemy")
   -- spawnBullet(hero_x, hero_y, speed, 262, "enemy")
   group.spawnBullet(hero_x, hero_y, speed, 260, "enemy")
   group.spawnBullet(hero_x, hero_y, speed, 250, "enemy")
  end ,
  inUse = false, bulletsFiredOff = 0
  }

  group.enemyWeapon = {}

  -- This is for the first enemy.  
  group.enemyWeapon[1] = { fireWeapon = function(enemy_x, enemy_y)
   group.spawnBullet(enemy_x, enemy_y, 230, 90, "hamster")
  end
  }

  -- These bullets are for the enemies that move in from the left
  -- and right sides of the screen.
  group.enemyWeapon[2] = {
   fireWeapon = function(enemy_group_2_x,enemy_group_2_y, speed, angle, target)
     group.spawnBullet(enemy_group_2_x, enemy_group_2_y, speed, angle, target)
   end
  }

  -- These bullets are for the third wave of enemies.  They just fly by,
  -- shooting at the hero.  The thing with this one is that they have to
  -- know where the hero is.
  group.enemyWeapon[3] = {
   fireWeapon = function(enemy_x, enemy_y, hamster_trajectory)
     group.spawnBullet(enemy_x, enemy_y, 240, hamster_trajectory , "hamster")
   end
  }

  -- Taken from the game SHMUP.
  function group.draw_bullets()
  -- Draw bullets
    for i,b in pairs(group.bullet) do
     if b.dead == false then  -- Need to use this attribute eventually.
       if b.who == "hamster" then
	 -- Don't really know what to do here...
       elseif b.who == "enemy" then
       end
       -- These are interesting formulas.  If you try using an
       -- angle of 90° you'll see the bullets move down,
       -- whereas if you gave it an angle of 270° the bullets will
       -- move up.
       local bx2 = b.x + (__cosine(b.angle-180) * PI_DIVIDED_BY_180) * (25)
       local by2 = b.y + (__sine(b.angle-180) * PI_DIVIDED_BY_180) * (25)

       -- Adding 64 to bx2 and by2 centers the bullet
       -- on the pictures.
       love.graphics.circle("fill", bx2+64, by2+64, 5, 100)
       
       -- Move the point used for collision detection.
       if b.circle then
	 b.circle:moveTo(bx2, by2)
       end
     end

   end
  end

  -- Code taken from the game SHMUP.
  function group.animate_bullets(changeInTime)
    for i,b in pairs(group.bullet) do
     if b.dead == false then
       b.x = group.moveBulletOnXaxis(b, changeInTime)
       b.y = group.moveBulletOnYaxis(b, changeInTime)  -- <-- Still need to work out the kinks in this one.
       love.graphics.circle("fill", b.x, b.y, 5, 100)
     end
    end
  end

  -- Move the bullets on the x-axis
  function group.moveBulletOnXaxis(bullet, dt)
    return bullet.x + (__cosine(bullet.angle * PI_DIVIDED_BY_180) * (bullet.speed * dt))
  end

  -- Move the bullets on the y-axis
  function group.moveBulletOnYaxis(bullet, dt)
    return bullet.y + (__sine(bullet.angle * PI_DIVIDED_BY_180) * (bullet.speed * dt))
  end

  -- Clean up any spent bullets.
  function group.cleanUpBullets()
    for i,b in pairs(group.bullet) do
      if b.y <= 0 then
	table.remove(group.bullet[i])
      elseif b.y >= love.graphics.getHeight() then
	table.remove(group.bullet[i])
      end
    end
  end

  return group
end